from imagescraperwithbrowser.location import Location
from logdecorator import log_on_start
from selenium import webdriver
from warnings import warn
from logging import INFO


class Browser:
    def __init__(self, config: dict = None):
        self._driver = webdriver.Chrome()
        self.config = config if config is not None else {}

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, value: dict):
        supported_config = ['disrespect_robot_rules', 'scroll_step']

        if not isinstance(value, dict):
            raise SystemExit('Configuration should be in dict format.')

        for c in value.keys():
            if c not in supported_config:
                raise SystemExit('Unknown configuration defined: {}'.format(c))

            if c in ['disrespect_robot_rules'] and not isinstance(value[c], bool):
                raise SystemExit('"{}" must be boolean.'.format(c))

        if 'disrespect_robot_rules' not in value.keys():
            value['disrespect_robot_rules'] = False

        if 'scroll_step' not in value.keys():
            value['scroll_step'] = 15

        self._config = value

    @property
    def body_scroll_height(self):
        return self._driver.execute_script('return document.body.scrollHeight')

    @property
    def body_scroll_top(self):
        return self._driver.execute_script('return window.scrollY')

    @property
    def user_agent(self):
        return self._driver.execute_script('return navigator.userAgent')

    @log_on_start(INFO, 'Closing browser')
    def close(self):
        self._driver.close()

    @log_on_start(INFO, 'Going to {location.url:s}')
    def go_to(self, location: Location):
        if not location.is_secure:
            warn('Your connection to "{}" may not be secure.'.format(location.hostname), stacklevel=2)

        robot_rules = location.robot_rules(self.user_agent)

        if not robot_rules['can_fetch'] and not self.config['disrespect_robot_rules']:
            self.close()
            raise SystemExit('"{}/{}" does not allow robots.'.format(location.hostname, location.pathname))

        self._driver.get(url=location.url)

    @log_on_start(INFO, 'Going to {data:s}')
    def go_to_data(self, data):
        self._driver.get(url=data)

    @log_on_start(INFO, 'Scrolling to {height:d}')
    def scroll_to(self, height: int):
        scroll_now = self.body_scroll_top
        scroll_to = height

        while scroll_now < scroll_to:
            self._driver.execute_script('window.scrollTo(0, {})'.format(scroll_now))
            scroll_now += self.config['scroll_step']

    def elements(self, value: str = None, identifier: str = None):
        supported_identifiers = ['tag', 'class']

        if identifier is 'tag':
            return self._driver.find_elements_by_tag_name(value)
        if identifier is 'class':
            return self._driver.find_elements_by_class_name(value)

        raise SystemExit('Identifier can be one of {}.'.format(', '.join(supported_identifiers)))



from imagescraperwithbrowser.location import Location
from imagescraperwithbrowser.browser import Browser
from logdecorator import log_on_start
from logging import INFO
from time import sleep
from tqdm import tqdm


class Scraper:
    def __init__(self, location: Location, browser: Browser, config: dict = None):
        self._location = location
        self._browser = browser
        self.config = config or {}

    @property
    def config(self):
        return self._config

    @config.setter
    def config(self, value: dict):
        supported_config = ['img_min_height', 'img_min_width', 'save_dir', 'max_img_count', 'sleep_time']

        if not isinstance(value, dict):
            raise SystemExit('Configuration should be in dict format.')

        for c in value.keys():
            if c not in supported_config:
                raise SystemExit('Unknown configuration defined: {}'.format(c))

            if c in ['img_min_height', 'img_min_width'] and not isinstance(value[c], int):
                raise SystemExit('"{}" must be integer.'.format(c))

            if c in ['save_dir'] and not isinstance(value[c], str):
                raise SystemExit('"{}" must be string.'.format(c))

            if c in ['sleep_time'] and not isinstance(value[c], (int, float)):
                raise SystemExit('"{}" must be numerical.'.format(c))

        if 'save_dir' not in value.keys():
            value['save_dir'] = 'images'
        else:
            value['save_dir'] = value['save_dir'].replace(' ', '_')

        self._config = value

    @log_on_start(INFO, 'Finding images')
    def find_images(self):
        images = self._browser.elements('img', 'tag')

        if 'img_min_height' in self._config:
            images = list(filter(lambda image: image.get_property('height') > self._config['img_min_height'], images))
        if 'img_min_width' in self._config:
            images = list(filter(lambda image: image.get_property('height') > self._config['img_min_width'], images))

        return images

    @log_on_start(INFO, 'Saving images')
    def save_images(self, images):
        from urllib.request import urlretrieve
        from os.path import isdir
        from os import makedirs

        if not isdir(self.config['save_dir']):
            makedirs(self.config['save_dir'])

        images = images[:self.config['max_img_count']]
        for index, image in enumerate(tqdm(images)):
            image_src = image.get_attribute('src') or image.get_attribute('data-src')
            image_name = '{save_dir}/{i}.png'.format(save_dir=self.config['save_dir'], i=index)

            try:
                urlretrieve(image_src, image_name)
                sleep(self.config['sleep_time'])
            except:
                continue

    def run(self):
        self._browser.go_to(location=self._location)

        images = []
        body_scroll_height = 0

        while len(images) < self.config['max_img_count']:
            images = self.find_images()

            body_scroll_height_new = self._browser.body_scroll_height
            if body_scroll_height_new > body_scroll_height:
                body_scroll_height = body_scroll_height_new
                self._browser.scroll_to(body_scroll_height)
            else:
                break

            sleep(self.config['sleep_time'])

        self.save_images(images)

        self._browser.close()



from urllib.parse import urlencode
from urllib.robotparser import RobotFileParser


class Location:
    def __init__(self, hostname: str, pathname: str = '', params: dict = None, protocol: str = 'https'):
        self.hostname = hostname
        self.pathname = pathname
        self.protocol = protocol

        self.params = params or {}

    @property
    def hostname(self):
        return self._hostname

    @hostname.setter
    def hostname(self, value: str):
        if not isinstance(value, str):
            raise SystemExit('Hostname should be string.')

        self._hostname = value

    @property
    def pathname(self):
        return self._pathname

    @pathname.setter
    def pathname(self, value: str):
        if not isinstance(value, str):
            raise SystemExit('Pathname should be string.')

        self._pathname = value

    @property
    def params(self):
        return self._params

    @params.setter
    def params(self, value: dict):
        if not isinstance(value, dict):
            raise SystemExit('Parameters should be in dict format.')

        self._params = value

    @property
    def protocol(self):
        return self._protocol

    @protocol.setter
    def protocol(self, value: str):
        allowed_protocols = ['http', 'https']

        if value not in allowed_protocols:
            raise SystemExit('Protocol can be one of {}.'.format(', '.join(allowed_protocols)))

        self._protocol = value

    @property
    def url(self):
        return '{protocol}://{hostname}/{pathname}?{params}'.format(
            protocol=self.protocol,
            hostname=self.hostname,
            pathname=self.pathname,
            params=urlencode(self.params)
        )

    @property
    def url_robots(self):
        return '{protocol}://{hostname}/robots.txt'.format(
            protocol=self.protocol,
            hostname=self.hostname,
        )

    @property
    def is_secure(self):
        return self.protocol is 'https'

    def robot_rules(self, user_agent: str):
        robot_parser = RobotFileParser(url=self.url_robots)
        robot_parser.read()

        return {
            'can_fetch': robot_parser.can_fetch(user_agent, self.url),
            'crawl_delay': robot_parser.crawl_delay(user_agent),
            'request_rate': robot_parser.request_rate(user_agent),
        }



from distutils.core import setup

setup(
    name='imagescraperwithbrowser',
    packages=['imagescraperwithbrowser'],
    version='0.1',
    license='gpl-3.0',
    description='Scrap images from websites with browser',
    author='Egehan Dulger',
    url='https://github.com/egehandulger/image-scrapper-with-browser',
    download_url='https://github.com/egehandulger/image-scraper-with-browser/archive/v0.1.tar.gz',
    keywords=['image', 'search', 'scrap', 'save', 'download'],
    install_requires=['selenium', 'tqdm', 'logdecorator'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
